class MixerSetCls
{
public:
	MixerSetCls();
	virtual ~MixerSetCls();
	void ActivateMic();

private:
	UINT m_nNumMixers;
	HMIXER m_hMixer;

	DWORD m_dwControlType, m_dwSelectControlID, m_dwMultipleItems, m_dwIndex, m_dwVolumeControlID, m_dwMaxVol;

	void amdUninitialize();
	BOOL amdInitialize();
	BOOL amdGetMicSelectControl();
	BOOL amdSetMicVolumeValue(DWORD dwVal) const;
	BOOL amdSetMicSelectValue(LONG lVal) const;
};

MixerSetCls::MixerSetCls()
{
	amdInitialize();
}

MixerSetCls::~MixerSetCls()
{
	amdUninitialize();
}

void MixerSetCls::ActivateMic()
{
	if (amdGetMicSelectControl()) {
		amdSetMicSelectValue(TRUE);
		amdSetMicVolumeValue(1);
	}
}

BOOL MixerSetCls::amdInitialize()
{
	m_nNumMixers = ::mixerGetNumDevs();

	m_hMixer = NULL;
	m_dwControlType = 0;
	m_dwSelectControlID = 0;
	m_dwVolumeControlID = 0;
	m_dwMaxVol = 0;
	m_dwMultipleItems = 0;
	m_dwIndex = ULONG_MAX;

	if (m_nNumMixers != 0) 
		if (::mixerOpen(&m_hMixer, 0, 0, NULL, MIXER_OBJECTF_MIXER) != MMSYSERR_NOERROR) 
			return FALSE;

	return TRUE;
}

void MixerSetCls::amdUninitialize()
{
	if (m_hMixer != NULL) {
		::mixerClose(m_hMixer);
		m_hMixer = NULL;
	}
}

BOOL MixerSetCls::amdGetMicSelectControl()
{
	DWORD dwi;

	if (m_hMixer == NULL)
		return FALSE;

	// dwLineID
	MIXERLINE mxl;
	mxl.cbStruct = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

	if (::mixerGetLineInfo(reinterpret_cast<HMIXEROBJ>(m_hMixer),
						   &mxl,
						   MIXER_OBJECTF_HMIXER |
						   MIXER_GETLINEINFOF_COMPONENTTYPE)
		!= MMSYSERR_NOERROR)
		return FALSE;

	// dwControlID
	MIXERCONTROL mxc;
	MIXERLINECONTROLS mxlc;
    ZeroMemory(&mxc, sizeof(mxc));
    mxc.cbStruct = sizeof(mxc);
	m_dwControlType = MIXERCONTROL_CONTROLTYPE_MIXER;
	mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID = mxl.dwLineID;
	mxlc.dwControlType = m_dwControlType;
	mxlc.cControls = 1;
	mxlc.cbmxctrl = sizeof(MIXERCONTROL);
	mxlc.pamxctrl = &mxc;
	if (::mixerGetLineControls(reinterpret_cast<HMIXEROBJ>(m_hMixer),
							   &mxlc,
							   MIXER_OBJECTF_HMIXER |
							   MIXER_GETLINECONTROLSF_ONEBYTYPE)
		!= MMSYSERR_NOERROR) {
		m_dwControlType = MIXERCONTROL_CONTROLTYPE_MUX;
		mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
		mxlc.dwLineID = mxl.dwLineID;
		mxlc.dwControlType = m_dwControlType;
		mxlc.cControls = 1;
		mxlc.cbmxctrl = sizeof(MIXERCONTROL);
		mxlc.pamxctrl = &mxc;
		if (::mixerGetLineControls(reinterpret_cast<HMIXEROBJ>(m_hMixer),
								   &mxlc,
								   MIXER_OBJECTF_HMIXER |
								   MIXER_GETLINECONTROLSF_ONEBYTYPE)
			!= MMSYSERR_NOERROR)
		{
			return FALSE;
		}
	}

	m_dwSelectControlID = mxc.dwControlID;
	m_dwMultipleItems = mxc.cMultipleItems;

	if (m_dwMultipleItems == 0)
		return FALSE;

	MIXERCONTROLDETAILS_LISTTEXT *pmxcdSelectText = new MIXERCONTROLDETAILS_LISTTEXT[m_dwMultipleItems];

	if (pmxcdSelectText != NULL) {
		MIXERCONTROLDETAILS mxcd;
		mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID = m_dwSelectControlID;
		mxcd.cChannels = 1;
		mxcd.cMultipleItems = m_dwMultipleItems;
		mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
		mxcd.paDetails = pmxcdSelectText;
		if (::mixerGetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer),
									 &mxcd,
									 MIXER_OBJECTF_HMIXER |
									 MIXER_GETCONTROLDETAILSF_LISTTEXT)
			== MMSYSERR_NOERROR) {
			for (dwi = 0; dwi < m_dwMultipleItems; dwi++) {
				MIXERLINE mxl;
				mxl.cbStruct = sizeof(MIXERLINE);
				mxl.dwLineID = pmxcdSelectText[dwi].dwParam1;
				if (::mixerGetLineInfo(reinterpret_cast<HMIXEROBJ>(m_hMixer),
									   &mxl,
									   MIXER_OBJECTF_HMIXER |
									   MIXER_GETLINEINFOF_LINEID)
					== MMSYSERR_NOERROR &&
					mxl.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE) {
					m_dwIndex = dwi;
					break;
				}
			}

			if (dwi >= m_dwMultipleItems) {
				for (dwi = 0; dwi < m_dwMultipleItems; dwi++) {
					if (::lstrcmp(pmxcdSelectText[dwi].szName, "Microphone") == 0) {
						m_dwIndex = dwi;
						break;
					}
				}
			}

			if (m_dwIndex < m_dwMultipleItems) {
				// Prende il controllo voulme del microfono
				ZeroMemory(&mxc, sizeof(mxc));
				mxc.cbStruct = sizeof(mxc);
	
				ZeroMemory(&mxlc, sizeof(mxlc));
				mxlc.cbStruct = sizeof(MIXERLINECONTROLS);
				mxlc.dwLineID = pmxcdSelectText[m_dwIndex].dwParam1;
				mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
				mxlc.cControls = 1;
				mxlc.cbmxctrl = sizeof(MIXERCONTROL);
				mxlc.pamxctrl = &mxc;
				if (::mixerGetLineControls(reinterpret_cast<HMIXEROBJ>(m_hMixer),
									   &mxlc,
									   MIXER_OBJECTF_HMIXER |
									   MIXER_GETLINECONTROLSF_ONEBYTYPE) == MMSYSERR_NOERROR) {
					m_dwVolumeControlID = mxlc.pamxctrl->dwControlID;
					m_dwMaxVol = mxlc.pamxctrl->Bounds.lMaximum;
				}
			}
		}

		delete []pmxcdSelectText;
	}

	return m_dwIndex < m_dwMultipleItems;
}


BOOL MixerSetCls::amdSetMicSelectValue(LONG lVal) const
{
	if (m_hMixer == NULL ||
		m_dwMultipleItems == 0 ||
		m_dwIndex >= m_dwMultipleItems)
		return FALSE;

	BOOL bRetVal = FALSE;

	MIXERCONTROLDETAILS_BOOLEAN *pmxcdSelectValue = new MIXERCONTROLDETAILS_BOOLEAN[m_dwMultipleItems];

	if (pmxcdSelectValue != NULL) {
		MIXERCONTROLDETAILS mxcd;
		mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID = m_dwSelectControlID;
		mxcd.cChannels = 1;
		mxcd.cMultipleItems = m_dwMultipleItems;
		mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
		mxcd.paDetails = pmxcdSelectValue;
		if (::mixerGetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer),
									 &mxcd,
									 MIXER_OBJECTF_HMIXER |
									 MIXER_GETCONTROLDETAILSF_VALUE)
			== MMSYSERR_NOERROR)
		{
			if (lVal != 0 && m_dwControlType == MIXERCONTROL_CONTROLTYPE_MUX) {
				::ZeroMemory(pmxcdSelectValue,
							 m_dwMultipleItems * sizeof(MIXERCONTROLDETAILS_BOOLEAN));
			}

			// setta il valore
			pmxcdSelectValue[m_dwIndex].fValue = lVal;

			mxcd.cbStruct = sizeof(MIXERCONTROLDETAILS);
			mxcd.dwControlID = m_dwSelectControlID;
			mxcd.cChannels = 1;
			mxcd.cMultipleItems = m_dwMultipleItems;
			mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
			mxcd.paDetails = pmxcdSelectValue;
			if (::mixerSetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer),
										 &mxcd,
										 MIXER_OBJECTF_HMIXER |
										 MIXER_SETCONTROLDETAILSF_VALUE)
				== MMSYSERR_NOERROR) {
				bRetVal = TRUE;
			}
		}

		delete []pmxcdSelectValue;
	}

	return bRetVal;
}


BOOL MixerSetCls::amdSetMicVolumeValue(DWORD dwVal) const
{
	if (m_hMixer == NULL || m_dwVolumeControlID == 0)
		return FALSE;

	MIXERCONTROLDETAILS mxcd;
	MIXERCONTROLDETAILS_UNSIGNED vol;
	ZeroMemory(&mxcd, sizeof(mxcd));
	mxcd.cbStruct = sizeof(mxcd);

    mxcd.dwControlID = m_dwVolumeControlID;
    mxcd.cbStruct = sizeof(mxcd);
    mxcd.cMultipleItems = 0;
    mxcd.cbDetails = sizeof(vol);
    mxcd.paDetails = &vol;
    mxcd.cChannels = 1;
    
	if (dwVal)
		vol.dwValue = m_dwMaxVol;
	else
		vol.dwValue = 0;

    mixerSetControlDetails(reinterpret_cast<HMIXEROBJ>(m_hMixer), &mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
}




